@@ -1,4 +1,3 @@
-import pysam
 
 class BAM():
     def __init__(self, filename, title=''):
@@ -7,7 +6,6 @@ class BAM():
             self.title = title
         else:
             self.title = self.filename.split('/')[-1]
-        self.samAlign = pysam.AlignmentFile(self.filename, "rb")
 
     def __str__(self):
         return self.title
