@@ -7,7 +7,7 @@ from zipfile import ZipFile
 from .drawreadset import DrawReadSet, CoveragePlot, CoverageHeatmap
 from .coordinates import COORDINATES
 from .geneplot import GenePlot
-from .baseplot import BasePlot
+from .basetrack import BaseTrack
 from .scale import Xscale
 from .bam import BAM
 from .util import get_url, getTemplatePath, fileOpen, fileSave, check_dir, getrgb, get_scale, is_exist
@@ -190,7 +190,8 @@ class BamSnap():
 
         check_dir(outfname)
         if self.opt['imagetype'] == "jpg":
-            ia.save(outfname, "JPEG", quality=1, optimize=True)
+            ia = ia.convert("RGB")
+            ia.save(outfname, "JPEG", quality=100, optimize=True)
         else:
             ia.save(outfname, "PNG", quality=1, optimize=True)
 
@@ -216,8 +217,7 @@ class BamSnap():
         font = self.get_font(fontsize, 'bold')
         fontsize = font.getsize(title)
         h = fontsize[1] + 3 + margin * 2
-        im = Image.new('RGBA', (w, h), getrgb(self.opt['bgcolor']))
-        # im = Image.new('RGBA', (w, h), getrgb("F000F0"))
+        im = Image.new('RGB', (w, h), getrgb(self.opt['bgcolor']))
         dr = ImageDraw.Draw(im)
 
         if self.opt['border']:
@@ -236,14 +236,14 @@ class BamSnap():
         return ia
 
     def get_image_seperator(self, w, h):
-        im = Image.new('RGBA', (w, h), getrgb(self.opt['bgcolor']))
+        im = Image.new('RGB', (w, h), getrgb(self.opt['bgcolor']))
         dr = ImageDraw.Draw(im)
         h1 = int(h/2)
         dr.line([(0, h1), (w, h1)], fill=getrgb('000000'), width=1)
         return im
 
     def init_image(self, image_w, bgcolor="FFFFFF"):
-        ia = Image.new('RGBA', (image_w, 0), getrgb(bgcolor))
+        ia = Image.new('RGB', (image_w, 0), getrgb(bgcolor))
         return ia
 
     def get_bamplot_image(self, bam, pos1, image_w, xscale, refseq):
@@ -252,6 +252,7 @@ class BamSnap():
         rset.read_gap_h = self.opt['read_gap_height']
         rset.read_thickness = self.opt['read_thickness']
         rset.coverage_vaf = self.opt['coverage_vaf']
+        rset.opt = self.opt
         rset.xscale = xscale
         rset.calculate_readmap(is_strand_group=True)
 
@@ -271,6 +272,7 @@ class BamSnap():
 
             if plot1 == "coverage":
                 covplot = CoveragePlot(rset, xscale, self.opt['coverage_vaf'])
+                covplot.coverage_color = self.opt['coverage_color']
                 covplot.font = self.get_font(self.opt['coverage_fontsize'])
                 ia_sub = covplot.get_image(image_w, self.opt['coverage_height'], self.opt['coverage_bgcolor'])
                 im = self.append_image(im, ia_sub)
@@ -278,18 +280,21 @@ class BamSnap():
             if plot1 == "read":
                 if self.opt['read_group'] == "":
                     h_all = rset.get_estimated_height('all')
-                    ia_sub = rset.get_image(image_w, h_all, 'all', self.opt['read_color'], self.opt['read_bgcolor'])
+                    ia_sub = rset.get_image(
+                        image_w, h_all, 'all', self.opt['read_color'], self.opt['read_bgcolor'], self.opt['read_color_by'])
                     im = self.append_image(im, ia_sub)
 
                 elif self.opt['read_group'] == "strand":
                     # self.opt['read_bgcolor'] = "F0F000"
                     h_pos = rset.get_estimated_height('pos_strand') 
-                    ia_sub = rset.get_image(image_w, h_pos, 'pos_strand', self.opt['read_pos_color'], self.opt['read_bgcolor'])
+                    ia_sub = rset.get_image(image_w, h_pos, 'pos_strand',
+                                            self.opt['read_pos_color'], self.opt['read_bgcolor'], self.opt['read_color_by'])
                     im = self.append_image(im, ia_sub)
 
                     # self.opt['read_bgcolor'] = "00F0F0"
                     h_neg = rset.get_estimated_height('neg_strand') 
-                    ia_sub = rset.get_image(image_w, h_neg, 'neg_strand', self.opt['read_neg_color'], self.opt['read_bgcolor'])
+                    ia_sub = rset.get_image(image_w, h_neg, 'neg_strand',
+                                            self.opt['read_neg_color'], self.opt['read_bgcolor'], self.opt['read_color_by'])
                     im = self.append_image(im, ia_sub)
 
             if plot1 == "coordinates":
@@ -334,7 +339,7 @@ class BamSnap():
         return ia
 
     def append_baseplot_image(self, ia, pos1, image_w, xscale, refseq):
-        baseplot = BasePlot(pos1['chrom'], pos1['g_spos'], pos1['g_epos'], refseq, xscale, image_w)
+        baseplot = BaseTrack(pos1['chrom'], pos1['g_spos'], pos1['g_epos'], refseq, xscale, image_w)
         baseplot.font = self.get_font(self.opt['base_fontsize'])
         ia_sub = baseplot.get_image(self.opt['base_margin_top'], self.opt['base_margin_bottom'])
         ia = self.append_image(ia, ia_sub)
@@ -350,6 +355,8 @@ class BamSnap():
 
             if plot1 == "bamplot":
                 for bidx, bam in enumerate(bamlist):
+                    if (bidx + 1) % 10 == 0:
+                        self.opt['log'].info("..processing " + bam.filename + " (" + str(bidx + 1) + ")")
                     ia_sub = self.get_bamplot_image(bam, pos1, image_w, xscale, refseq)
                     ia = self.append_image(ia, ia_sub)
 
@@ -362,11 +369,17 @@ class BamSnap():
 
             if plot1 == "base":
                 ia = self.append_baseplot_image(ia, pos1, image_w, xscale, refseq)
-            
+
+        if self.opt['grid'] > 0:
+            drawA = ImageDraw.Draw(ia)
+            for posi in range(pos1['g_spos'], pos1['g_epos'] + 1, self.opt['grid']):
+                x1 = xscale.xmap[posi]['spos'] - 1
+                drawA.line([(x1, 0), (x1, ia.height)], fill=COLOR['GRID_COLOR'], width=1)
+
         if self.opt['center_line']:
             drawA = ImageDraw.Draw(ia)
             x1 = ((self.opt['g_epos']-self.opt['g_spos'])/2)*self.scale_x
-            for h1 in range(0, image_h, 7):
+            for h1 in range(0, ia.height, 7):
                 drawA.line([(x1, h1), (x1, h1+2)], fill=COLOR['CENTER_LINE'], width=1)
 
         if not self.opt['no_target_line']:
@@ -412,7 +425,7 @@ class BamSnap():
     
     def start_process_drawplot(self, image_w, bamlist):
         for tno in range(self.opt['process']):
-            self.process[tno] = mp.Process(target=self.start_process_drawplot_bamlist, args=(image_w, bamlist,self.split_poslist[tno]), name='process ' + str(tno))
+            self.process[tno] = mp.Process(target=self.start_process_drawplot_bamlist, args=(image_w, bamlist, self.split_poslist[tno]), name='proc ' + str(tno+1))
             self.process[tno].start()
 
     def start_process_drawplot_bamlist(self, image_w, bamlist, poslist):
